==> testcafe-legacy-api-3.1.6.tgz: package/lib/compiler/tools/uglify-js/lib/squeeze-more.js <==
"use strict";

var jsp = require("./parse-js"),
    pro = require("./process"),
    slice = jsp.slice,
    member = jsp.member,
    curry = jsp.curry,
    MAP = pro.MAP,
    PRECEDENCE = jsp.PRECEDENCE,
    OPERATORS = jsp.OPERATORS;

function ast_squeeze_more(ast) {
        var w = pro.ast_walker(),
            walk = w.walk,
            scope;
        function with_scope(s, cont) {
                var save = scope,
                    ret;
                scope = s;
                ret = cont();
                scope = save;
                return ret;
        };
        function _lambda(name, args, body) {
                return [this[0], name, args, with_scope(body.scope, curry(MAP, body, walk))];
        };
        return w.with_walkers({
                "toplevel": function (body) {
                        return [this[0], with_scope(this.scope, curry(MAP, body, walk))];
                },
                "function": _lambda,
                "defun": _lambda,
                "new": function (ctor, args) {
                        if (ctor[0] == "name") {
                                if (ctor[1] == "Array" && !scope.has("Array")) {
                                        if (args.length != 1) {
                                                return ["array", args];
                                        } else {
                                                return walk(["call", ["name", "Array"], args]);
                                        }
                                } else if (ctor[1] == "Object" && !scope.has("Object")) {
                                        if (!args.length) {
                                                return ["object", []];
                                        } else {
                                                return walk(["call", ["name", "Object"], args]);
                                        }
                                } else if ((ctor[1] == "RegExp" || ctor[1] == "Function" || ctor[1] == "Error") && !scope.has(ctor[1])) {
                                        return walk(["call", ["name", ctor[1]], args]);
                                }
                        }
                },
                "call": function (expr, args) {
                        if (expr[0] == "dot" && expr[1][0] == "string" && args.length == 1 && (args[0][1] > 0 && expr[2] == "substring" || expr[2] == "substr")) {
                                return ["call", ["dot", expr[1], "slice"], args];
                        }
                        if (expr[0] == "dot" && expr[2] == "toString" && args.length == 0) {
                                // foo.toString()  ==>  foo+""
                                return ["binary", "+", expr[1], ["string", ""]];
                        }
                        if (expr[0] == "name") {
                                if (expr[1] == "Array" && args.length != 1 && !scope.has("Array")) {
                                        return ["array", args];
                                }
                                if (expr[1] == "Object" && !args.length && !scope.has("Object")) {
                                        return ["object", []];
                                }
                                if (expr[1] == "String" && !scope.has("String")) {
                                        return ["binary", "+", args[0], ["string", ""]];
                                }
                        }
                }
        }, function () {
                return walk(pro.ast_add_scope(ast));
        });
};

exports.ast_squeeze_more = ast_squeeze_more;

==> testcafe-legacy-api-4.1.0.tgz: package/lib/compiler/tools/uglify-js/lib/squeeze-more.js <==
"use strict";

var jsp = require("./parse-js"),
    pro = require("./process"),
    slice = jsp.slice,
    member = jsp.member,
    curry = jsp.curry,
    MAP = pro.MAP,
    PRECEDENCE = jsp.PRECEDENCE,
    OPERATORS = jsp.OPERATORS;

function ast_squeeze_more(ast) {
        var w = pro.ast_walker(),
            walk = w.walk,
            scope;
        function with_scope(s, cont) {
                var save = scope,
                    ret;
                scope = s;
                ret = cont();
                scope = save;
                return ret;
        };
        function _lambda(name, args, body) {
                return [this[0], name, args, with_scope(body.scope, curry(MAP, body, walk))];
        };
        return w.with_walkers({
                "toplevel": function toplevel(body) {
                        return [this[0], with_scope(this.scope, curry(MAP, body, walk))];
                },
                "function": _lambda,
                "defun": _lambda,
                "new": function _new(ctor, args) {
                        if (ctor[0] == "name") {
                                if (ctor[1] == "Array" && !scope.has("Array")) {
                                        if (args.length != 1) {
                                                return ["array", args];
                                        } else {
                                                return walk(["call", ["name", "Array"], args]);
                                        }
                                } else if (ctor[1] == "Object" && !scope.has("Object")) {
                                        if (!args.length) {
                                                return ["object", []];
                                        } else {
                                                return walk(["call", ["name", "Object"], args]);
                                        }
                                } else if ((ctor[1] == "RegExp" || ctor[1] == "Function" || ctor[1] == "Error") && !scope.has(ctor[1])) {
                                        return walk(["call", ["name", ctor[1]], args]);
                                }
                        }
                },
                "call": function call(expr, args) {
                        if (expr[0] == "dot" && expr[1][0] == "string" && args.length == 1 && (args[0][1] > 0 && expr[2] == "substring" || expr[2] == "substr")) {
                                return ["call", ["dot", expr[1], "slice"], args];
                        }
                        if (expr[0] == "dot" && expr[2] == "toString" && args.length == 0) {
                                // foo.toString()  ==>  foo+""
                                return ["binary", "+", expr[1], ["string", ""]];
                        }
                        if (expr[0] == "name") {
                                if (expr[1] == "Array" && args.length != 1 && !scope.has("Array")) {
                                        return ["array", args];
                                }
                                if (expr[1] == "Object" && !args.length && !scope.has("Object")) {
                                        return ["object", []];
                                }
                                if (expr[1] == "String" && !scope.has("String")) {
                                        return ["binary", "+", args[0], ["string", ""]];
                                }
                        }
                }
        }, function () {
                return walk(pro.ast_add_scope(ast));
        });
};

exports.ast_squeeze_more = ast_squeeze_more;
